/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h.
 * Restrictions in GB_copyright.h.
 *
 *	makeplanet.c -- makes one planet.  used with makeuniv
 */

#include "GB_copyright.h"

#define MAP_ISLANDS 10		/* # of beginning islands for makeuniv */
#define MAP_MOUNT_PERCENT 0.22	/* percentage of mountain areas */
#define MAP_GASGIANT_BANDMIN 20  /* min size gasgiants that have bands */
#define LANDPERCENTAGE 0.45

#define EARTHTYPE_RANGE int_rand(36,39)
#define AIRLESS_RANGE int_rand(int_rand(16,20),25)
#define GASGIANT_RANGE int_rand(27,MAX_X-10)
#define ASTEROID_RANGE_X int_rand(int_rand(2,4),6)	/* asteroids have */
#define ASTEROID_RANGE_Y int_rand(int_rand(2,5),8)	/*  funny shapes */
#define ICEBALL_RANGE int_rand(int_rand(18,23),26)
#define WATER_RANGE int_rand(20,MAX_X-15)

#define RES_GAS int_rand(20,40)
#define RES_SEA int_rand(60,90)
#define RES_LAND int_rand(60,90)
#define RES_MOUNT int_rand(90,120)
#define RES_ICE int_rand(50,70)

#define EXTERN extern
#include "vars.h"
#include <math.h>
long random();
double Distmap();	/* (different distmap from the shell one) */


planettype Makeplanet(dist, stemp, type)
float dist;
short stemp;
int type;
{
reg int x,y;
sectortype *s;
planettype planet;
int i, atmos,landsects,Numlandsects;

  Bzero(planet);
  bzero((char *)Smap, sizeof(Smap));
  planet.type = type;
  planet.expltimer = 5;

switch (type) {
 case TYPE_GASGIANT:		/* gas giant planet */

  planet.Maxx = GASGIANT_RANGE;
  planet.Maxy = round_rand((float)planet.Maxx / RATIOXY) + 1;

 		/* change to more real range */
  planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);
	/* either lots of meth or not too much */
  if (random()&01) {	/* methane planet */
    atmos = 100 - (planet.conditions[METHANE] = int_rand(70,80)); 
    atmos -= planet.conditions[HYDROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[HELIUM] = 1;
    atmos -= planet.conditions[OXYGEN] = 0;
    atmos -= planet.conditions[CO2] = 1;
    atmos -= planet.conditions[NITROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = atmos;
  } else {
    atmos = 100 - (planet.conditions[HYDROGEN] = int_rand(30,55));
    atmos -= planet.conditions[HELIUM] = int_rand(30,atmos/2);
    atmos -= planet.conditions[METHANE] = random()&01;
    atmos -= planet.conditions[OXYGEN] = 0;
    atmos -= planet.conditions[CO2] = random()&01;
    atmos -= planet.conditions[NITROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = atmos;
  }

  for (y=0; y<planet.Maxy; y++)
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des = DES_GAS;	/* for gasgiant */
	s->fert = int_rand(35,55);
	s->resource = RES_GAS;
     }
 if (planet.Maxx > MAP_GASGIANT_BANDMIN) {
    int nbands;
    nbands = MAX(1,planet.Maxy/5);
    for (i=1; i<=nbands; i++) {
     y=int_rand(1,planet.Maxy-2);
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des = DES_SEA;	/* make random bands of water */
	s->fert += int_rand(20,30);
	s->resource = RES_SEA;
     }
    }
    for (i=0; i<=MAP_ISLANDS*2; i++) {
     y=int_rand(1,planet.Maxy-2);
     x=int_rand(1,planet.Maxx-2);
	s = &Sector(planet,x,y);
	s->des = DES_SEA;	/* make random spots of water */
	s->fert += int_rand(60,90);
	s->resource = RES_SEA;
    }
 } else {
    for (i=0; i<MAP_ISLANDS*2; i++) {
	s = &Sector(planet, int_rand(1,planet.Maxx-2), int_rand(1,planet.Maxy-2));
	if (random()&01) {
		s->des=DES_MOUNT;
		s->is_wasted = 1;
		s->resource = RES_MOUNT;
		s->fert = int_rand(10,21);
	} else {
		s->des=DES_SEA;
		s->fert=int_rand(60,90);
		s->resource = RES_SEA;
	}
    }
  Smashup(&planet,int_rand(2,4),DES_SEA,0,0,0);
  Smashup(&planet,int_rand(1,2),DES_LAND,2,3,1);
 }

 break;

case TYPE_AIRLESS:		 /* make airless wasted planet */

planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);
    planet.conditions[HYDROGEN] = 0;
    planet.conditions[HELIUM] = 0;
    planet.conditions[METHANE] = 0;
    planet.conditions[OXYGEN] = 0;
    atmos = 100 - (planet.conditions[CO2] = int_rand(30,45));
    atmos -= planet.conditions[NITROGEN] = int_rand(10, atmos/2);
    atmos-=planet.conditions[SULFUR] = random()&01 ? 0 : int_rand(20, atmos/2);
    atmos-=planet.conditions[OTHER] = atmos;

  planet.Maxx = AIRLESS_RANGE;
  planet.Maxy = round_rand(planet.Maxx / RATIOXY);
  for (y=0; y<planet.Maxy; y++)
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des = DES_MOUNT;	/* for airless */
	s->resource = RES_MOUNT;
	s->fert=int_rand(0,int_rand(3,10));
     }

     for (i=0; i<MAP_ISLANDS; i++) {
	s = &Sector(planet, int_rand(1,planet.Maxx-2), int_rand(1,planet.Maxy-2));
	s->des = DES_LAND;
	s->fert += int_rand(3,17);
	s->resource = RES_LAND;
     }
  Smashup(&planet,6,DES_LAND,5,1,0);	/* make meteor impacts */

  break;

case TYPE_ASTEROID: 		/* asteroid */

	/* no atmosphere */
    planet.conditions[HYDROGEN] = 0;
    planet.conditions[HELIUM] = 0;
    planet.conditions[METHANE] = 0;
    planet.conditions[OXYGEN] = 0;
    planet.conditions[CO2] = 0;
    planet.conditions[NITROGEN] = 0;
    planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = 0;
  planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);
  planet.Maxx = ASTEROID_RANGE_X;
  planet.Maxy = ASTEROID_RANGE_Y;
  for (y=0; y<planet.Maxy; y++)
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des = int_rand(0,3) ? DES_MOUNT : DES_LAND;
	s->is_wasted = 0;

	if(s->des==DES_MOUNT)
	  s->resource = RES_MOUNT;
	else
	  s->resource = RES_LAND;

	s->fert = 1 + random()&01;
     }

     break;
  
case TYPE_EARTH:

 planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);
  if (random()&01) {
	/* oxygen-reducing atmosphere (Earth type) */
    atmos = 100 - (planet.conditions[OXYGEN] = int_rand(10,25));
    atmos -= planet.conditions[NITROGEN] = int_rand(20,atmos/2);
    atmos -= planet.conditions[CO2] = int_rand(20,atmos/2);
    atmos -= planet.conditions[HELIUM] = int_rand(2,atmos/8+1);
    atmos -= planet.conditions[METHANE] = random()&01;
    atmos -= planet.conditions[SULFUR] = 0;
    atmos -= planet.conditions[HYDROGEN] = 0;
    planet.conditions[OTHER] = atmos;
  } else {
	/* methane atmosphere */
    atmos = 100 - (planet.conditions[METHANE] = int_rand(70,80)); 
    atmos -= planet.conditions[HYDROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[HELIUM] = 1;
    atmos -= planet.conditions[OXYGEN] = 0;
    atmos -= planet.conditions[CO2] = 1;
    atmos -= planet.conditions[NITROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = atmos;
  }
  planet.Maxx = EARTHTYPE_RANGE;
  planet.Maxy = round_rand(planet.Maxx/RATIOXY);   /* may haveta change this */
  Numlandsects = planet.Maxx*planet.Maxy*LANDPERCENTAGE;

  for (y=0; y<planet.Maxy; y++)
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des=DES_SEA;	/* make all water first */
	s->fert = int_rand(20,43);
	s->resource = RES_SEA;
     }
		/* make first isolated islands */
  for (i=0; i<=MAP_ISLANDS; i++) {
     s = &Sector(planet, int_rand(planet.Maxx/5,planet.Maxx-planet.Maxx/5),
	     int_rand(1,planet.Maxy-2));
     s->des = DES_LAND;  
     s->resource = RES_LAND;
     s->fert += int_rand(6,20);
  }

  /*PermuteSects(&planet);	/* mix up map */
  /*(void)Getxysect(&planet,0,0,1);	/* reset index */

  landsects=0;
  while (landsects<=Numlandsects)
       landsects += Volcano(&planet,landsects,Numlandsects,1); 
		   /* pick rndm locn, return whether island or not */


		/* make more isolated islands */
  for (i=0; i<=MAP_ISLANDS*3; i++) {
     x=int_rand(0,planet.Maxx-1);
     y=int_rand(0,planet.Maxy-1);
     s = &Sector(planet,x,y);
     s->des = DES_LAND;
     s->resource = RES_LAND;	/* nice odd number */
		  /* min content prop to dist from sea */
     s->fert = 45;
  }

   break;

case TYPE_WATER:
 planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);
  if (random()&01) {
	/* oxygen-reducing atmosphere (Earth type) */
    atmos = 100 - (planet.conditions[OXYGEN] = int_rand(10,25));
    atmos -= planet.conditions[NITROGEN] = int_rand(20,atmos/2);
    atmos -= planet.conditions[CO2] = int_rand(20,atmos/2);
    atmos -= planet.conditions[HELIUM] = int_rand(2,atmos/8+1);
    atmos -= planet.conditions[METHANE] = random()&01;
    atmos -= planet.conditions[SULFUR] = 0;
    atmos -= planet.conditions[HYDROGEN] = 0;
    planet.conditions[OTHER] = atmos;
  } else {
	/* methane atmosphere */
    atmos = 100 - (planet.conditions[METHANE] = int_rand(70,80)); 
    atmos -= planet.conditions[HYDROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[HELIUM] = 1;
    atmos -= planet.conditions[OXYGEN] = 0;
    atmos -= planet.conditions[CO2] = 1;
    atmos -= planet.conditions[NITROGEN] = int_rand(1,atmos/2);
    atmos -= planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = atmos;
  }
  planet.Maxx = WATER_RANGE;
  planet.Maxy = round_rand(planet.Maxx/RATIOXY);   /* may haveta change this */
  Numlandsects = planet.Maxx*planet.Maxy*LANDPERCENTAGE;

  for (y=0; y<planet.Maxy; y++)
     for (x=0; x<planet.Maxx; x++) {
	s = &Sector(planet,x,y);
	s->des=DES_SEA;	/* make all water first */
	s->fert = int_rand(20,43);
	s->resource = RES_SEA;
     }

landsects=0;
  while (landsects<=Numlandsects)
       landsects += Volcano(&planet,landsects,Numlandsects,0); 
		   /* pick rndm locn, return whether island or not */

    break;

case TYPE_ICEBALL:		/* ball of ice */
	/* no atmosphere */
    planet.conditions[HYDROGEN] = 0;
    planet.conditions[HELIUM] = 0;
    planet.conditions[METHANE] = 0;
    planet.conditions[OXYGEN] = 0;
    planet.conditions[CO2] = 0;
    planet.conditions[NITROGEN] = 0;
    planet.conditions[SULFUR] = 0;
    planet.conditions[OTHER] = 0;
 planet.conditions[TEMP] = planet.conditions[RTEMP] = Temperature(dist, stemp);

    planet.Maxx = ICEBALL_RANGE;
    planet.Maxy = round_rand(planet.Maxx / RATIOXY);

    for (y=0; y<planet.Maxy; y++)
       for (x=0; x<planet.Maxx; x++) {
	    Sector(planet,x,y).des = DES_ICE;
	    Sector(planet,x,y).resource = RES_ICE;
	    Sector(planet,x,y).fert = int_rand(10,25);
       }
 
     /* make some random mountains */
    for (i=0; i<MAP_ISLANDS; i++) {
	x = int_rand(0,planet.Maxx-1);
	y = int_rand(0,planet.Maxy-1);
	Sector(planet,x,y).des = DES_MOUNT;
	Sector(planet,x,y).resource = RES_MOUNT;
	Sector(planet,x,y).fert *= 0.3;
    }
    Smashup(&planet, 3, DES_MOUNT, 20, 0, 0);
       
    break;

default:
    break;

 }

 return planet;

}


	/* smash psuedo-meteors into the planet */
Smashup(pptr,n,desig,res,fert,waste)	
planettype *pptr;
int n;			/* # iterations */
char desig;		/* des to make target sectors */
int res,fert,waste;	/* resources,fert to add, whether to waste */
{
reg int r,x,y;
reg int x2,y2;
reg sectortype *s;

 while (n--) {
    x=int_rand(1,pptr->Maxx-2);
    y=int_rand(1,pptr->Maxy-2);
    r=int_rand(2,int_rand(2,pptr->Maxy/3+1));
    for (y2=y-r-1; y2<=y+r+1; y2++)
       for (x2=x-r-1; x2<=x+r+1; x2++)
	  if (Distmap(x,y,x2,y2) < r) 
	     if ((x2<pptr->Maxx)&&(x2>=0)&&(y2<pptr->Maxy)&&(y2>=0)) {
		s = (&(Smap[y2*pptr->Maxx+x2]));
		if (desig==DES_LAND && s->des==DES_MOUNT)
			s->des = DES_LAND;
		else if (desig==DES_LAND && s->des==DES_LAND)
			s->des = DES_MOUNT;
		if (waste) s->is_wasted=1;
		if (res) s->resource+=int_rand(res/2,res*2);
		if (fert) s->fert+=int_rand(fert/2,fert*2);
	     }
 }

}



double Distmap(x,y,x2,y2)
reg int x,y,x2,y2;
{
 return fabs((double)x-x2) / RATIOXY + fabs( (double)y-y2);
}


int Volcano(pptr,landsectors,Numlandsects,continent)
planettype *pptr;
int landsectors,Numlandsects,continent;
{
reg int x,y;
sectortype *s;

  x = int_rand(0,pptr->Maxx-1);
  y = int_rand(0,pptr->Maxy-1);
  s = &Sector(*pptr,x,y);

    if (y > pptr->Maxy/2) {
      if (y==pptr->Maxy-1) {
	s->des = DES_ICE;
	return(random()&01);
      } else {
	if ((Sector(*pptr,x,y+1).des==DES_ICE) && 
		 (int_rand(-50,10)>pptr->conditions[RTEMP])) {
		s->des = DES_ICE;
		return(random()&01);
	}
      }
    } else {
      if (y==0) {
	s->des = DES_ICE;
	return(random()&01);
      } else if ((Sector(*pptr,x,y-1).des==DES_ICE) && 
		 (int_rand(-50,10)>pptr->conditions[RTEMP])) {
		s->des = DES_ICE;
		return(random()&01);
	}
    }

  if (s->des==DES_SEA && continent) {
    if (Numneighbors(pptr,x,y,DES_LAND)||Numneighbors(pptr,x,y,DES_MOUNT)) {
	s->des=(landsectors>(MAP_MOUNT_PERCENT*Numlandsects))?DES_LAND:DES_MOUNT;
	s->resource = round_rand((float)(Numlandsects - landsectors + 5) / (float)Numlandsects * 100);
		  /* min content prop to dist from sea */
	s->fert = (float)(landsectors + 5) / (float)Numlandsects * 100;
		  /* fert content prop to dist from center of continent */
       return(1);
    } else return(0);
   } else return(0);
}

int Temperature(dist, stemp)
float dist;
short stemp;
{
float factor;
	factor = 20.0*((float)stemp*(1000./dist)-6.0);
	return ((int)factor);
} 

/* return # of neighbors of a given designation that a sector has. */
int Numneighbors(p,x,y,des)
planettype *p;
reg int x,y,des;
{
reg int d;
  return (Sector(*p,mod(x-1,p->Maxx,d),y).des==des) + 
  	 (Sector(*p,mod(x+1,p->Maxx,d),y).des==des) + 
  	 ((y==0) ? 0 : (Sector(*p,x,y-1).des==des)) + 
  	 ((y==p->Maxy-1) ? 0 : (Sector(*p,x,y+1).des==des));
}
