/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h.
 * Restrictions in GB_copyright.h.
 *
 * shoot() -- shoot from ship or planet to ship or planet (does dam acc. to
 * 		range -- does not remove dest cap from firing ship/planet)
 * blast() -- generic damaging routine -- saves only smap
 * tele_range(r,s) -- range of telescope
 * gun_range(r,s) -- range of ship's guns
 */

#include "GB_copyright.h"
#define EXTERN extern
#include "vars.h"
#include "ships.h"
#include "races.h"
#include "buffers.h"
#include <math.h>
extern int Distmap();
int shoot_racedata, shoot_shdata;

float blast();
float Num_hits();

int shoot(Playernum,from,to,fromplayer,fpl,tpl,sectorx,sectory,strength,dist, 
	tech, msg, getmap)
int Playernum;
placetype from,*to;
int fromplayer;
planettype *fpl,*tpl;
int sectorx,sectory,strength;
float *dist,tech;
char *msg;
int getmap;
{
shiptype *ship;
double xfrom,yfrom,xto,yto;
float nd;
int casualties;
 sprintf(msg,""); /* 'zero' it out */
 if (from.level==LEVEL_PLAN) {
   xfrom = Stars[from.snum]->xpos + fpl->xpos;
   yfrom = Stars[from.snum]->ypos + fpl->ypos;
 } else if (from.level==LEVEL_SHIP) {
   if (from.shipptr->is_docked && from.shipptr->whatdest==LEVEL_PLAN) {
   	xfrom = Stars[from.shipptr->storbits]->xpos + fpl->xpos;
   	yfrom = Stars[from.shipptr->storbits]->ypos + fpl->ypos;
	if (to->level==LEVEL_SHIP &&
	    to->shipptr->is_docked && to->shipptr->whatdest==LEVEL_PLAN) {
		 /* ground distance */
		xfrom += from.shipptr->xpos;
		yfrom += from.shipptr->ypos;
	}
   } else {
   	xfrom = from.shipptr->xpos;
   	yfrom = from.shipptr->ypos;
   }
 }

 if (to->level==LEVEL_PLAN) {
   xto = Stars[to->snum]->xpos + tpl->xpos;
   yto = Stars[to->snum]->ypos + tpl->ypos;
 } else if (to->level==LEVEL_SHIP) {
   if (to->shipptr->is_docked && to->shipptr->whatdest==LEVEL_PLAN) {
   	xto = Stars[to->shipptr->storbits]->xpos + tpl->xpos;
   	yto = Stars[to->shipptr->storbits]->ypos + tpl->ypos;
	if (from.level==LEVEL_SHIP &&
	    from.shipptr->is_docked && from.shipptr->whatdest==LEVEL_PLAN) {
		xto += to->shipptr->xpos;
		yto += to->shipptr->ypos;
	}
   } else {
   	xto = to->shipptr->xpos;
   	yto = to->shipptr->ypos;
   }
 }

 *dist = sqrt( (double)Distsq(xfrom, yfrom, xto, yto));

 /* for ship-planet combat: check whether the attacking ship 
    is close enough, if not do not fire */

   if ( (*dist > PLORBITSIZE) &&
       (from.level==LEVEL_SHIP) && (to->level==LEVEL_PLAN))

    { 
      sprintf(buf,"Ship is not close enough to attack the planet !!\n");
		notify(Playernum, buf);
      return -1;
    }


 if (*dist > SYSTEMSIZE) {
	 /* out of range */
	return -1;

 } else {

     nd = blast(Playernum,&from, to, fromplayer, tpl, sectorx, sectory, *dist, tech,strength, getmap);

     if (from.level==LEVEL_SHIP && from.shipptr->type==OTYPE_GR)
	sprintf(buf,"%.0f points radiation damage given\n", nd);
     else if (to->level==LEVEL_SHIP) {
		racetype *alien;
		sprintf(buf,"%.0f%% total damage done\n", nd);

	str_cat(msg, buf);

		casualties = 0;
		if(to->shipptr->popn) {
		/* evaluate crew casualties */
		  casualties = int_rand(0,
				       round_rand(.01*nd * to->shipptr->popn));
		  if (casualties > to->shipptr->popn)
		    casualties = to->shipptr->popn;

		  to->shipptr->popn -= casualties;
		openracedata(&shoot_racedata);
		  getrace(shoot_racedata, &alien, to->shipptr->owner);
		close_file(shoot_racedata);
		  to->shipptr->mass -= casualties * alien->mass;
		  free(alien);
		  if (casualties) {
		    sprintf(buf,"%d crew casualties\n", casualties);
		str_cat(msg, buf);
		  }
	        }
     } else {
	    sprintf(buf,"%.0f sectors destroyed\n", nd);
	str_cat(msg, buf);
	  }

     if (to->level==LEVEL_SHIP && !to->shipptr->is_alive) {
	sprintf(buf,"%s #%d %s DESTROYED.\n", Shipnames[to->shipptr->type],
			to->shipno,to->shipptr->name);
	str_cat(msg, buf);
     }

     return (int)nd;
 }

}


	/* returns amt of damage or # of sectors nuked */

float blast(Playernum,from, to, fromplayer, pl, x, y, dist, tech, strength, getmap)
int Playernum;
placetype *from,*to;
int fromplayer;		/* shot came from what player */
planettype *pl;		/* pointer to planet being destroyed */
int x,y;		/* sector x,y of planet */
float dist;
float tech;       /* distance to target , technology of attacking guns*/
int strength;		/* strength of blast */
int getmap;		/* whether to get smap or not */
{
register int x2,y2;
reg float r;		/* radius of blast */
float d,damage,str;
int numdest = 0,lowx,lowy,hix,hiy,crithits,critdam,dam;
int arm,body,fevade,tevade,fspeed,tspeed;
int i, col, row, die, effect, kills;
register sectortype *s;
shiptype *ship;
sectortype *s2;
register float fac;
int shfdata,sectdata;
int checkcrit = 0;

 if (to->level==LEVEL_SHIP) {
 
	fevade = 0;
	tevade = 0;
	fspeed = 0;
	tspeed = 0;

	if(from->level==LEVEL_SHIP) 
			if((from->shipptr->whatdest || from->shipptr->navigate.on) && !from->shipptr->is_docked && from->shipptr->active)
			{ 
			fspeed=from->shipptr->speed;
			fevade=from->shipptr->protect.evade;
			} 

	 if((to->shipptr->whatdest  || to->shipptr->navigate.on) && !to->shipptr->is_docked && to->shipptr->active)
	 		{
				tspeed = to->shipptr->speed;
	 			tevade=to->shipptr->protect.evade;
			}
	
	body =  Size(to->shipptr);

	str = Num_hits(Playernum,dist,strength,tech,fevade,tevade,fspeed,tspeed,body);
	
	arm  =  Armor(to->shipptr);
	sprintf(buf,"NUMBER OF HITS: %.0f    ARMOR: %d    BODY: %d \n",
			str,arm,body);
		notify(Playernum, buf);
    
	numdest = round_rand (40.*((float)str - (float)arm)/(float)body) *
		(1 + 2*(to->shipptr->type == STYPE_MIRROR)); /*mirrors are easier to damage */ 

	if(numdest < 0)
			numdest = 0;
	
	sprintf(buf,"%d %% structural damage inflicted! \n",numdest);
		notify(Playernum, buf);
/* evaluate critical hits */
	checkcrit = 1;

	if(from->level == LEVEL_SHIP) {
		if((from->shipptr->type == OTYPE_GR) )
			checkcrit = 0;
	}


		if(checkcrit)
		{
		critdam = 0;
		crithits = 0;
		for(i=1; i<=str; i++)
			if(!int_rand(0, 15*body))						
				{
					crithits++;
					dam=int_rand(0,100);
					critdam += dam;
				}

		if(critdam > 100)critdam = 100;
		if(crithits)
			{
				sprintf(buf,"%d CRITICAL HIT(S) for %d %% additional damage!!!\n",
						crithits,critdam);
		notify(Playernum, buf);
				numdest += critdam;
			}

		}

	if (numdest < 0)   numdest = 0;
	else
	if (numdest > 100) numdest = 100; 

	if (from->level==LEVEL_SHIP && from->shipptr->type==OTYPE_GR) {
		/* gamma-ray lasers irradiate targets */
			to->shipptr->rad = numdest;
			if(int_rand(1,100) <= numdest) 
				to->shipptr->active = 0;
	} else {

		if (to->shipptr->damage+numdest >= 100) {
			kill_ship(to->shipptr);
			 /* mess up the planet a bit. */
		if (to->shipptr->type==OTYPE_TOXWC &&
			    to->shipptr->whatorbits==LEVEL_PLAN)
			pl->conditions[TOXIC] = 
				MIN(100, pl->conditions[TOXIC]+to->shipptr->object.number);
/* if the ship is docked, undock the both of them - do damage to the other? */
if(to->shipptr->is_docked && to->shipptr->whatdest == LEVEL_SHIP) {
	    openshdata(&shoot_shdata); /* got to do this! */
	    getship(shoot_shdata, &ship, to->shipptr->destshipno);
	    ship->is_docked = 0;
	    ship->whatdest = LEVEL_UNIV;
	    ship->destshipno = 0;
	    putship(shoot_shdata, ship, to->shipptr->destshipno);
	    free(ship);
	    close_file(shoot_shdata);
		    to->shipptr->is_docked = 0; /* not really important...*/
	            to->shipptr->whatdest = LEVEL_UNIV;
	            to->shipptr->destshipno = 0;
	}
	     } else
	       to->shipptr->damage += numdest;
    }
    
	Nuked[to->shipptr->owner-1] = 1;
	return (float)numdest;		/* return amt of dest made */

 } else if (to->level==LEVEL_PLAN) {

	str = 2*strength;
    	r = .2 * str;

    if (getmap) {
	    opensectdata(&sectdata);
	    getsmap(sectdata,Smap,pl);
	    close_file(sectdata);
	}

    lowx = MAX((x-r-1),0);
    lowy = MAX((y-r-1),0);
    hix = MIN((x+r+1),pl->Maxx-1);
    hiy = MIN((y+r+1),pl->Maxy-1);
    for (y2=lowy; y2<=hiy; y2++) {
         for (x2=lowx; x2<=hix; x2++) {
	  if ( (d = Distmap(x,y,x2,y2) ) <= r) {
	     s = &Sector(*pl,x2,y2);
	     fac = ((float)str) * logscale(100 - s->mobilization)/(d+1);
		/* factor of destruction */
	    /* printf("%d,%d  dist = %.2f,dest fac %.2f \n",x2,y2,d,fac);*/
	     	if (fac > 1.0) {
		   	Nuked[s->owner-1] = 1;	/* for retaliating */
						
			kills = int_rand(0,((int)fac))/(1+(s->des==DES_PLATED));
		if(kills >= s->popn)
			{
			  s->popn = 0;
			  s->owner = 0;
			}
		else {
			s->popn -= kills;
		      }
	if(fac >= 5.0 && !int_rand(0,10))
		{
			if (s->des==DES_ICE)
				s->des=DES_SEA;
			else if (s->des==DES_PLATED)
				if(int_rand(0,100) > 30)s->des=DES_LAND;
			else if (s->des==DES_SEA)
				s->des=DES_LAND;
			else if (s->des==DES_MOUNT) 
				if(int_rand(0,100) > 20)s->des=DES_LAND;
			else if (s->des==DES_LAND && 
				(s+1)->des==DES_SEA || (s-1)->des==DES_SEA)
					s->des=DES_SEA;

			else if (s->des==DES_LAND && 
				(s+1)->des==DES_GAS || (s-1)->des==DES_GAS)
					s->des=DES_GAS;
		}
	     	   	if(fac >= 5.0)
				if(((float)int_rand(0,10))*fac > 50.0)
					{
				s->popn = 0;
				s->owner = 0;
				s->eff = 0;
				s->mobilization = 0;
				s->fert = 0; /*all is lost !*/
				s->is_wasted = 1;
				numdest++;
					}
/* not sure why this is necessary (gvc)
	     	   	s->resource = (percent)round_rand(s->resource * fac);
 		   	s->fert = (percent)round_rand(s->fert / fac);
	     	   	s->mobilization = (percent)(round_rand(s->mobilization / fac) );
	     	   	s->eff = (percent)round_rand(s->eff / fac); */
		}
	  }
	}
    }

/* planet toxicity goes up a bit */
    pl->conditions[TOXIC] += (100-pl->conditions[TOXIC]) * 
				((float)numdest / (pl->Maxx*pl->Maxy));

    if (getmap) {
	opensectdata(&sectdata);
	putsmap(sectdata,Smap,pl);
	    close_file(sectdata);
    }

   }
   return (float)numdest;

}


float Num_hits(Playernum,dist, guns,tech,fev,tev,fspeed,tspeed,body)
int Playernum;
float dist,tech;
int guns,fev,tev,fspeed,tspeed,body;
{
int factor;
int i,prob,hit_odds(),hits;

if(fev){
	sprintf(buf," firing ship EVADING! \n");
		notify(Playernum, buf);
	}
if(tev) {
	sprintf(buf,"target ship EVADING!\n");
		notify(Playernum, buf);
	 }
sprintf(buf,"firing speed: %d ",fspeed);
		notify(Playernum, buf);
sprintf(buf,"     target speed: %d     target size: %d \n",tspeed,body);
		notify(Playernum, buf);

hits=0;
prob = hit_odds(dist,&factor,tech,fev,tev,fspeed,tspeed,body);
sprintf(buf,"50%% range = %d      target range = %d\n",factor,(int)dist);
		notify(Playernum, buf);
sprintf(buf,"hit probability = %d%%\n",prob);
		notify(Playernum, buf);

for(i=1; i<=guns; i++)
	if(int_rand(1,100) <= prob) 
		hits++;

 return hits;
}

int hit_odds(range,factor,tech,fev,tev,fspeed,tspeed,body)
float range,tech;
int fev,tev,fspeed,tspeed,body;
int *factor;
{
int odds;
double a,b,c;

a = log1p((double)tech) * 80.0 * (float)body;
b = (2.0+(float)tev) * (2.0+(float)fev) * (3.0+(float)tspeed+(float)fspeed);
c = a/b;
*factor = (int)c;
odds = 0;
if(*factor > 0)
	odds = (int) ((float)((*factor )*100)/((float)((*factor)+(int)range)));

return odds;
}

/* 
 * range of telescopes, ground or space, given race and ship 
*/
float tele_range(type,tech) 
int type;
float tech;
{
  if (type==OTYPE_GTELE)
	return log1p((double)tech) * 400 + SYSTEMSIZE/8;
  else
	return log1p((double)tech) * 1500 + SYSTEMSIZE/3;
}



/* 
 * gun range of given ship, given race and ship 
 */
float gun_range(r,s)
reg racetype *r;
reg shiptype *s;
{
  return (logscale((int)(r->tech+1.0)) * SYSTEMSIZE);
}

